import { AxiosRequestConfig, Method } from 'axios';
import { API } from '../utils';

/* REQUEST_EXPORTS */

export const request = (
  method: Method,
  url: string,
  data: any = null,
  headers: any = null
) => {
  return API().request({ method, url, data, headers } as AxiosRequestConfig);
};

export interface Request {
  status: 'idle' | 'loading' | 'success' | 'failed';
}

export interface RequestErrors {
  status: number;
  title: string;
  detail: string;
}

export interface FailedRequest {
  errors: RequestErrors;
}

export interface GetRelationshipsData {
  id: number;
  type: string;
}

export interface GetRelationshipsRequest {
  data: GetRelationshipsData[];
}

export interface GetIncludedData<M> {
  type: string | null;
  id: number | null;
  attributes: M;
}

export interface GetData<M, R = null> {
  type?: string | null;
  id?: number | null;
  attributes: M;
  relationships?: M | null;
  included?: GetIncludedData<R>[] | null;
}

export interface GetAllRequest<M> {
  data: GetData<M>[];
}

export interface GetOneRequest<M> {
  data: GetData<M>;
}

export interface UpsertRelationshipsData<RL = null> {
  attributes?: RL | null;
}

export interface UpsertRelationshipsRequest<RR = null> {
  data?: UpsertRelationshipsData<RR> | null;
}

export interface UpsertData<MR, RR = null> {
  attributes: MR;
  relationships?: UpsertRelationshipsRequest<RR> | null;
}

export interface UpsertRequest<MR, RR = null> {
  data: UpsertData<MR, RR>;
}

export interface DeleteAttributes {
  success: boolean;
}

export interface DeleteData {
  attributes: DeleteAttributes;
}

export interface DeleteRequest {
  data: DeleteData;
}
